function XIMSSController() {
  var that = this;
  var ximssSession = null;
  var taskRef = null;

  function ximssAsyncProcessor(xmlResponse) {
    console.log('S: ximssAsyncProcessor');
    console.log(xmlResponse);
  }
  function ximssNetworkErrorProcessor(isFatal,timeElapsed) {
    console.log(isFatal ? 'fatal' : 'having problems for ' + timeElapsed + ' sec');
  }
  function readTaskEventResponse(xmlData) {
    if(xmlData.getAttribute('eventName') != 'report' || xmlData.getAttribute('taskRef') != taskRef) return xmlData;
    $('html').trigger('reportReceived',xmlData);
  }
  function loginCallback(newSession,errorCode) {
    if(errorCode != null) {
      console.log('S: loginCallback');
      console.log(errorCode);
      closeSession();
      return;
    }
    ximssSession = newSession;
    ximssSession.setUnknownAsyncProcessor(that,ximssAsyncProcessor);
    ximssSession.setNetworkErrorProcessor(that,ximssNetworkErrorProcessor);
    ximssSession.setAsyncProcessor(that,readTaskEventResponse,'taskEvent');
    ximssSession.start();
    that.sendRequest('taskFindMeeting',{meetingName:'conference'});
    return;
  }
  function logoutCallback(errorCode,xmlRequest) {
    if(errorCode != null) {
      console.log('S: logoutCallback');
      console.log(errorCode);
      console.log(xmlRequest);
    }
  }
  function dataCallback(xmlData,xmlRequest) {
    console.log('S: dataCallback');
    console.log(xmlData);
    if(xmlData.nodeName == 'taskCreated' || xmlData.nodeName == 'taskMeeting') {
      taskRef = xmlData.getAttribute('taskRef');
      if(taskRef != null) that.sendRequest('taskSendEvent',{taskRef:taskRef,eventName:'control'});
    }
  }
  function finalCallback(errorCode,xmlRequest) {
    if(errorCode != null) {
      console.log('S: finalCallback');
      console.log(errorCode);
      console.log(xmlRequest);
    }
  }

  this.getRef = function() {
    return taskRef;
  }
  this.resetRef = function() {
    taskRef = null;
  }
  this.createSession = function(serverAddress,userName,sessionID) {
    var params = new Object;
    params.binding = 'HTTP';
    if(location.href.substr(0,6) == 'https:') params.secureMode = 'YES';
    params.serverName = serverAddress;
    params.loginMethod = 'sessionid';
    params.userName = userName;
    params.sessionid = sessionID;
    params.pollPeriod = 20;
    new XIMSSSession(params,this,loginCallback);
    return;
  }
  this.closeSession = function() {
    if(ximssSession != null) {
      ximssSession.close(that,logoutCallback);
    }
  }
  this.sendRequest = function(request,params,data) {
    if(ximssSession == null) {
      alert('ximssSession does not exist');
      return;
    }
    xmlRequest = ximssSession.createXMLNode(request);
    xmlRequest.setAttribute('report','xml');
    if(params != null) {
      for (var attr in params) {
        xmlRequest.setAttribute(attr,params[attr]);
      }
    }
    if(data != null) {
      xmlRequest.appendChild(ximssSession.createTextNode(data));
    }
    console.log('C:');
    console.log(xmlRequest);
    ximssSession.sendRequest(xmlRequest,that,dataCallback,finalCallback,true);
  }
}

var myController = null;
var confRef = null;

function createSession(serverName,serverPort,userName,sessionID) {
  if(myController == null) {
    var serverAddress = serverName;
    if(serverPort != null) serverAddress += ':' + serverPort;
    myController = new XIMSSController();
    myController.createSession(serverAddress,userName,sessionID);
  } else console.log('D: createSession');
}
function startConference() {
  if(myController != null) {
    myController.sendRequest('taskStart',{programName:'conferenceStarter'});
  } else console.log('D: startConference');
}
function inviteParty(partyExt) {
  if(myController != null) {
    confRef = myController.getRef();
    if(confRef != null) {
      var confEvent = (partyExt == 'self') ? 'callSilently' : 'callOut';
      myController.sendRequest('taskSendEvent',{taskRef:confRef,eventName:confEvent},partyExt);
    } else console.log('D: inviteParty confRef');
  } else console.log('D: inviteParty');
}
function muteParty(partyID) {
  if(myController != null) {
    confRef = myController.getRef();
    if(confRef != null) {
      myController.sendRequest('taskSendEvent',{taskRef:confRef,eventName:'muteParty'},partyID);
    } else console.log('D: muteParty confRef');
  } else console.log('D: muteParty');
}
function unmuteParty(partyID) {
  if(myController != null) {
    confRef = myController.getRef();
    if(confRef != null) {
      myController.sendRequest('taskSendEvent',{taskRef:confRef,eventName:'unmuteParty'},partyID);
    } else console.log('D: unmuteParty confRef');
  } else console.log('D: unmuteParty');
}
function triggerMusic() {
  if(myController != null) {
    confRef = myController.getRef();
    if(confRef != null) {
      myController.sendRequest('taskSendEvent',{taskRef:confRef,eventName:'telCom'},'music');
    } else console.log('D: triggerMusic confRef');
  } else console.log('D: triggerMusic');
}
function disconnectParty(partyID) {
  if(myController != null) {
    confRef = myController.getRef();
    if(confRef != null) {
      myController.sendRequest('taskSendEvent',{taskRef:confRef,eventName:'disconnectParty'},partyID);
    } else console.log('D: disconnectParty confRef');
  } else console.log('D: disconnectParty');
}
function isStarted() {
  if(myController != null) {
    confRef = myController.getRef();
    if(confRef != null) return true;
    else return false;
  } else console.log('D: isStarted');
}
function stopConference() {
  if(myController != null) {
    confRef = myController.getRef();
    if(confRef != null) {
      myController.sendRequest('taskSendEvent',{taskRef:confRef,eventName:'telCom'},'#');
      myController.sendRequest('taskClose',{taskRef:confRef});
      myController.resetRef();
    }
    myController.sendRequest('taskRemoveMeeting',{meetingName:'conference'});
  } else console.log('D: stopConference');
}
function closeSession() {
  if(myController != null) {
    myController.closeSession()
  } else console.log('D: closeSession');
}
